import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

@SuppressWarnings({"unchecked", "deprecation"})

public class Questions extends GUI
{    
    public static void main(String[] args)
    {
        new Questions();
    }

    String slash = System.getProperty("file.separator");
    //  use / for Mac , \ for Windows
    String folder = System.getProperty("user.dir")+slash;
    // path+name of project folder

    Button next = addButton("Next",20,20,100,50,this); 
    Label question  = addLabel("",150,20,450,50,this);
    Button answer = addButton("Answer",150,80,400,50,this);
    Label score = addLabel("",20,80,100,50,this); 
    Button samples = addButton("Add 3",520,150,50,50,this);
    Button save = addButton("Save",520,200,50,50,this);
    Button load = addButton("Load",520,250,50,50,this);

    JLabel pic;               // will show a picture later

    int count = -1; 

    int points = 0;

    LinkedList<Question> qs = new LinkedList<Question>();
    
    public Questions()
    {
        super(600,600);
    }

    public void addSampleQuestions()
    {        
        Question newQ = new Question("Ronald MacDonald is a _______",
            "clown","clown.gif");
        qs.add(newQ);
        
        newQ = new Question("The capital of Germany is _____",
            "Berlin","Germany.png");
        qs.add(newQ);
        
        newQ = new Question("True/False: Cats like chocolate",
            "False","cats.jpg");
        qs.add(newQ);
    }

    public void saveData()
    {
        try
        {
            // Write to disk with FileOutputStream
            FileOutputStream fileOut = new 
                FileOutputStream(folder+"questions.data");

            // Serialize object with ObjectOutputStream
            ObjectOutputStream objOut = new
                ObjectOutputStream(fileOut);

            // Write object out to disk
            objOut.writeObject ( qs );     
        }
        catch(Exception ex)
        {
            output("ERROR = " + ex.toString());
        }
    }

    public void loadData()
    {
        try
        {
            // Read from disk using FileInputStream
            FileInputStream fileIn = new 
                FileInputStream(folder+"questions.data");

            // Unserialize object using ObjectInputStream
            ObjectInputStream objIn = 
                new ObjectInputStream (fileIn);

            // Read an object
            Object obj = objIn.readObject();

            // Cast to correct type and store in qs
            qs = (LinkedList<Question>)obj;   
        }
        catch(Exception ex)
        {
            output("ERROR = " + ex.toString()); 
        }
    }

    public void actions(Object source,String command)
    {
        if(source==next)                   
        {
            nextQuestion();
        } 

        if(source==answer)
        {
            getAnswer();
        } 

        if(source==save)
        {
            saveData();
        }

        if(source==load)
        {
            loadData(); 
        }
        
        if(source==samples)
        {
            addSampleQuestions();
        }
        
       
    }

    public void nextQuestion()
    {
        count = count + 1;             // counting next question
        Question ques = qs.get(count);
        
        if(pic!=null)                  // if pic contains a picture
        {pic.setVisible(false);}       //  erase it
        
        pic = addJLabel(ques.fileName,50,150,500,400,this);
        // load picture from project's folder
        pic.setVisible(true);  // show the picture
        question.setText(ques.questionText);  // show question
        answer.setLabel("Answer");           // write "answer" on button
    }

    public void getAnswer()
    {
        String guess = input("answer?");
        answer.setLabel(guess);          // write guess in Button
        if(guess.equalsIgnoreCase(qs.get(count).answerText))  // right answer?
        {
            answer.setBackground(new Color(150,255,150)); 
            // green means correct
            points = points + 100;      //  win 100  
        }
        else                            // wrong answer
        {
            answer.setBackground(new Color(255,150,150));
            // red means wrong
            points = points - 40;       //  lose 40
        }      
        score.setText(points + "");     // show points in label
    }
}

class Question implements Serializable
{
    String questionText = "";
    String answerText = "";
    String fileName = "";

    public Question()   // allow NEW without any parameters
    {
    }

    public Question(String q, String a, String f)
    {
        questionText = q;
        answerText = a;
        fileName = f;
    }
}

/****************************************************

=== LinkedList ===

This program demonstrates the use of a LinkedList
to store Question objects, where each Question object
contains questionText, answerText and fileName .

It also shows how to use Serialization to store
an Object into a data file on the hard-disk.
The Object could be a Question, containing 3 Strings,
or it can be the ENTIRE LINKEDLIST OF OBJECTS.

A LinkedList is similar to an array - but with one difference.
An Array has a STATIC size - maybe 100 Objects.
A LinkedList has a DYNAMIC size - that means it can
hold any number of Objects, and can GROW and SHRINK.
Java uses .add(Object) to add a new Object to the list.

After .add is used several times to add Objects to the list,
the program uses .get(position) to retrieve an Object.
So the following loop would display each Object:

  for(int c = 0; c < qs.size(); c++)
  {
     Question q = qs.get(c);
     output( q.questionText );
  }
  
Technically, a LinkedList is "unsafe".
That is because it is very easy to loop off the
end of the list, encountering null pointer exceptions.
There are also other possible errors involved
incorrect REFERENCES (pointers).  If we don't care
about errors (which beginners don't care about),
we can use the following to hide the warning messages:
  @SuppressWarnings({"unchecked", "deprecation"})

== Practice ==

(1) Download this program and run it.
    Click [add 3] to add the 3 sample questions.
    Click [save] to write them into a disk file.
    End the program.
    Run the program again.
    Now click[load] to retrieve the 3 questions that were saved.
    Check that [next] successfully displays the questions.
    Also check that the [answer] button works correctly.
    
(2) Try pressing [add 3] three times.
    Now there are 9 questions in the array (3 copies of 3).
    Save the 9 questions and then check that they load correctly.
    
(3) Make a new button called [New].  This should input
    a question, an answer and the name of a picture file.
    Then create a NEW Question Object, and copy the 3 Strings
    into the Object.  Finally, add the new Question Object
    to the qs array, and increase countQs.  After adding
    a few new questions then [save] them on the disk.
    And check that loading them again works correctly.
    
(4) Add a [RESET] button that resets back to the
    first Question.  This just requires changing
    the value of "count" back to 0 or -1 (find out which).
    
(5) Change the program so that whenever [Next] is clicked,
    it checks whether "count" is equal (or greater)
    to the value of qs.size().  That means all the questions
    have been used.  Then "count" should return to
    the beginning of the list, and the quiz can continue.
    
(6) Now we can add something new - after a question has
    been answered correctly, it is removed from the 
    list by using:
    
       qs.remove(count);
       
    Now it is no longer possible to answer the same
    question over and over again.
    
*****************************************************/
